import type { AnonCredsRegistry, GetCredentialDefinitionReturn, GetRevocationStatusListReturn, GetRevocationRegistryDefinitionReturn, GetSchemaReturn, RegisterCredentialDefinitionOptions, RegisterCredentialDefinitionReturn, RegisterSchemaOptions, RegisterSchemaReturn } from '@aries-framework/anoncreds';
import type { AgentContext } from '@aries-framework/core';
export declare class IndySdkAnonCredsRegistry implements AnonCredsRegistry {
    /**
     * This class supports resolving and registering objects with did:indy as well as legacy indy identifiers.
     * It needs to include support for the schema, credential definition, revocation registry as well
     * as the issuer id (which is needed when registering objects).
     */
    readonly supportedIdentifier: RegExp;
    getSchema(agentContext: AgentContext, schemaId: string): Promise<GetSchemaReturn>;
    registerSchema(agentContext: AgentContext, options: RegisterSchemaOptions): Promise<RegisterSchemaReturn>;
    getCredentialDefinition(agentContext: AgentContext, credentialDefinitionId: string): Promise<GetCredentialDefinitionReturn>;
    registerCredentialDefinition(agentContext: AgentContext, options: RegisterCredentialDefinitionOptions): Promise<RegisterCredentialDefinitionReturn>;
    getRevocationRegistryDefinition(agentContext: AgentContext, revocationRegistryDefinitionId: string): Promise<GetRevocationRegistryDefinitionReturn>;
    getRevocationStatusList(agentContext: AgentContext, revocationRegistryId: string, timestamp: number): Promise<GetRevocationStatusListReturn>;
    private fetchIndySchemaWithSeqNo;
}
