import type { IndySdk } from '../types';
import type { FileSystem, Logger } from '@aries-framework/core';
import type { LedgerReadReplyResponse, LedgerRequest, LedgerWriteReplyResponse } from 'indy-sdk';
import type { Subject } from 'rxjs';
export interface TransactionAuthorAgreement {
    version: `${number}.${number}` | `${number}`;
    acceptanceMechanism: string;
}
export interface IndySdkPoolConfig {
    /**
     * Optional id that influences the pool config that is created by the indy-sdk.
     * Uses the indyNamespace as the pool identifier if not provided.
     */
    id?: string;
    genesisPath?: string;
    genesisTransactions?: string;
    isProduction: boolean;
    indyNamespace: string;
    transactionAuthorAgreement?: TransactionAuthorAgreement;
    connectOnStartup?: boolean;
}
export declare class IndySdkPool {
    private indySdk;
    private logger;
    private fileSystem;
    private poolConfig;
    private _poolHandle?;
    private poolConnected?;
    authorAgreement?: AuthorAgreement | null;
    constructor(poolConfig: IndySdkPoolConfig, indySdk: IndySdk, logger: Logger, stop$: Subject<boolean>, fileSystem: FileSystem);
    get didIndyNamespace(): string;
    get id(): string | undefined;
    get config(): IndySdkPoolConfig;
    close(): Promise<void>;
    delete(): Promise<void>;
    connect(): Promise<void>;
    private connectToLedger;
    private submitRequest;
    submitReadRequest(request: LedgerRequest): Promise<LedgerReadReplyResponse>;
    submitWriteRequest(request: LedgerRequest): Promise<LedgerWriteReplyResponse>;
    private getPoolHandle;
    private getGenesisPath;
}
export interface AuthorAgreement {
    digest: string;
    version: string;
    text: string;
    ratification_ts: number;
    acceptanceMechanisms: AcceptanceMechanisms;
}
export interface AcceptanceMechanisms {
    aml: Record<string, string>;
    amlContext: string;
    version: string;
}
